/**
 * 异步包装
 * @packageDocumentation
 * @module vx-tools/asyncFormat
 */
/**
 * async 异步包装
 * @param { promise } promise 被包装promise
 * @returns { pormise } 包装后的promise
 * @summary 将 aysnce await 错误作为返回值的处理方式
 *
 * @example
 *  asynce load(){
 *
 *      const [ res, err ] = await asyncFormat( api(...) )
 *
 *      if(res === null){
 *        console.log(err)
 *        return
 *      }
 *
 *  }
 */
async function asyncFormat(promise) {
    return promise.then(res => [res, null]).catch((err) => [null, err]);
}

/**
 * 常用工具函数
 * @packageDocumentation
 * @module vx-tools/joinUrlParams
 */
/**
 * 拼接路由参数
 * @param { object } obj 参数对象
 * @example
 * ``` ts
 *  const parmas = {
 *    id: 1,
 *    name: 'Co'
 *  }
 *
 *  const url = joinUrlParams('/users', params)
 *  // url = /users?id=1&name=Co
 * ```
 */
function joinUrlParams(url, obj) {
    if (!obj) {
        return url;
    }
    return Object.entries({ ...obj }).reduce((acc, [key, value], index) => `${acc}${index ? '&' : '?'}${key}=${value}`, url);
}

/**
 * 对象过滤器
 * @packageDocumentation
 * @module vx-tools/objectFilter
 */
function objectFilter(keys, source) {
    return keys.reduce((acc, next) => {
        return {
            ...acc,
            [next]: source[next]
        };
    }, {});
}

/**
 * 延时器
 * @packageDocumentation
 * @module vx-tools/delay
 */
/**
 * 延时器
 * @summary
 * 通过promise 将函数推入任务队列，延时执行
 * @example
 * ``` ts
 *   // 模式一
 *   // 利用 async 等待delay执行完成, 此时delay将向任务队列中插叙空函数
 *   async function load(){
 *     // 延时获取数据
 *
 *     await delay(2)
 *     const data = await API.getData(...)
 *     ...
 *   }
 *
 *   // 模式二
 *   // 直接将回调函数作物，delay的延时执行函数
 *   delay(3, () => API.getData(...)).then(data => console.log('data: ', data))
 *
 * ```
 */
function delay(time, cb) {
    return new Promise((reslove) => {
        setTimeout(() => {
            reslove(cb ? cb() : null);
        }, time * 1000);
    });
}

/**
 * 本地缓存包装函数
 * @packageDocumentation
 * @module vx-tools/localSave
 */
let registed = false;
/**
 * 本地缓存包装函数
 *
 * @summary
 * 为本地缓存对象 localStorage，sessionStorage 添加 setItem getItem 包装函数
 * 包装函数将原字符参数或返回值通过JSON转换为对象
 *
 * @remind
 * 因为使用JSON对数据做转换, 数据格式必须服务json格式, 所以报错纯字符时将报错,
 * 此时可以使用原生方法
 *
 * @example
 * ``` ts
 * import { registSaveFn } from '@micro/utils'
 * registSaveFn() //全局调用一次
 *
 * const saveKey = 'USER'
 * localStorage.$setItem(saveKey, { name: 'copy', sex: 'man' })
 * const user = localStorage.$getItem(saveKey)
 * console.log(user.name)
 * ```
 */
function registSaveFn() {
    if (registed) {
        return;
    }
    localStorage.__proto__.$setItem = buildSetItem(localStorage);
    localStorage.__proto__.$getItem = buildGetItem(localStorage);
    sessionStorage.__proto__.$setItem = buildSetItem(sessionStorage);
    sessionStorage.__proto__.$getItem = buildGetItem(sessionStorage);
    registed = true;
}
function buildSetItem(storage) {
    return (key, data) => {
        try {
            const dataStr = JSON.stringify(data);
            storage.setItem(key, dataStr);
        }
        catch (error) {
            console.error(`${typeof (storage)} 设置失败: `, error);
            return null;
        }
    };
}
function buildGetItem(storage) {
    return (key) => {
        const dataStr = storage.getItem(key);
        try {
            return dataStr ? JSON.parse(dataStr) : null;
        }
        catch (error) {
            console.error(`${typeof (storage)} 获取失败: `, error);
            return null;
        }
    };
}

/**
 * 延时器
 * @packageDocumentation
 * @module vx-tools/Clock
 */
/**
 * 计时器
 * @summary
 * 记录一段或多段时间间隔
 *
 * @example
 * ```ts
 *  const c = new Clock // or Clock.create()
 *
 *  c.tick()
 *  setTimeout(() => {
 *    c.tick()
 *  }, 1000)
 *
 *  setTimeout(() => {
 *   console.log('time:', c.nextTick())
 *  }, 2000)
 *
 * ```
 */
class Clock {
    // 迭代器终止标识
    static END = 'END';
    // 工厂函数
    static create() {
        return new Clock();
    }
    _times = [];
    _currentTick = 0;
    // 获取记录列表
    _getTimes() {
        return [...this._times];
    }
    /**
     * 打点记录
     */
    tick() {
        this._times.push((new Date()).getTime());
    }
    /**
     * 清空记录
     */
    clean() {
        this._times = [];
    }
    /**
     * 记录条数
     * @returns number
     */
    len() {
        return this._times.length;
    }
    /**
     * 获取记录间隔
     * @param start
     * @param end
     * @returns
     */
    getTick(start, end) {
        const startTime = this._times[start];
        const endTime = this._times[end];
        if (!startTime || !endTime) {
            throw new Error(`查询越界: ${start} - ${end}`);
        }
        return endTime - startTime;
    }
    /**
     * 最后一条间隔
     * @returns number
     */
    lastTick() {
        return this.getTick(this._times.length - 2, this._times.length - 1);
    }
    /**
     * 第一条间隔
     * @returns number
     */
    firstTick() {
        return this.getTick(0, 1);
    }
    /**
     * 每条间隔迭代器
     * @returns number
     */
    nextTick() {
        if (this._currentTick >= this._times.length - 1) {
            return Clock.END;
        }
        const current = this._currentTick;
        const next = this._currentTick += 1;
        return this.getTick(current, next);
    }
    /**
     * 重置迭代器指针
     */
    resetTick() {
        this._currentTick = 0;
    }
    /**
     * 当前迭代器指针位置
     */
    currentTick() {
        return this._currentTick;
    }
}

/**
 * 断言
 *  - isString() - 字符类型
 *  - isNumber() - 数字类型
 *  - isBoolean() - 布尔类型
 *  - isUndefined() - undefined
 *  - isNull() - null
 *  - isFunction() - 函数类型
 *  - isObject() - 对象类型
 *  - isArray() - 数组类型
 *
 *  - isEventName() - 是否为事件名称 例如: 'onClick'
 *
 *  - isType() - 是否为某中类型 例如: isType('xx', 'string')
 *  - type() - 获取数据类型 例如: type('xxx') // string
 *
 * @example
 * ``` ts
 *  isString('xx')
 *  // => true
 * ```
 * @packageDocumentation
 * @module vx-tools/assert
 */
/**
 * 获取对象类型字符
 * @param data
 * @returns 类型字符
 */
function type(data) {
    const type = Object.prototype.toString.call(data);
    return type.replace(/\[object|\]|\s/igm, '').toLocaleLowerCase();
}
/**
 * 类型校验
 * @param data
 * @param typeStr
 */
function isType(data, typeStr) {
    return type(data) === typeStr;
}
// 基础类型
const isString = (data) => type(data) === 'string';
const isNumber = (data) => type(data) === 'number';
const isBoolean = (data) => type(data) === 'boolean';
const isUndefined = (data) => data === undefined;
const isNull = (data) => data === null;
const isArray = (data) => type(data) === 'array';
const isFunction = (data) => type(data) === 'function';
const isObject = (data) => type(data) === 'object';
// 特定类型
const isEventName = (data) => (/^on[A-Z]+[a-z]+$/).test(data);
const isSlotName = (data) => (/^v-slot(\:{1}[a-zA-Z]+)?$/).test(data);

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  type: type,
  isType: isType,
  isString: isString,
  isNumber: isNumber,
  isBoolean: isBoolean,
  isUndefined: isUndefined,
  isNull: isNull,
  isArray: isArray,
  isFunction: isFunction,
  isObject: isObject,
  isEventName: isEventName,
  isSlotName: isSlotName
});

export { Clock, index as assert, asyncFormat, buildGetItem, buildSetItem, delay, joinUrlParams, objectFilter, registSaveFn };
